from typing import Dict

from pyspark.sql import SparkSession, DataFrame
import logging

logger = logging.getLogger()


def createSession(name: str, spark_jars: Dict[str, str], mongodb_uri: str, starrok_uri: str) -> SparkSession:
    session = None
    try:
        jars = list(spark_jars.values())
        jars = ",".join(jars)
        session = SparkSession.builder \
            .appName(name) \
            .config("spark.hadoop.fs.s3a.impl", "org.apache.hadoop.fs.s3a.S3AFileSystem") \
            .config("spark.hadoop.fs.s3a.aws.credentials.provider",
                    "com.amazonaws.auth.DefaultAWSCredentialsProviderChain") \
            .config("spark.jars", jars) \
            .config("spark.executor.extraClassPath", jars) \
            .config("spark.driver.extraClassPath", jars) \
            .config("spark.mongodb.input.uri", mongodb_uri) \
            .config("spark.mongodb.output.uri", mongodb_uri) \
            .getOrCreate()
            # .config("spark.starrocks.url", starrok_uri) \
            # .config("spark.starrocks.driver", "com.starroks.jdbc.Driver") \
            # .config("spark.sql.catalogImplementation", "in-memory") \
            # .getOrCreate()
        session._jsc.hadoopConfiguration().set("fs.s3a.impl", "org.apache.hadoop.fs.s3a.S3AFileSystem")
    except Exception as e:
        logger.error(f"Error creando sesion. {e}")
    finally:
        return session


def get_goal_by_kpi(df: DataFrame, agent: str, period: str, kpi: str) -> float:
    result = 0.0
    try:
        df = df.filter((df["CEDULA"] == agent) & (df["PERIODO_PROCESO_CODIGO"] == period) & (df["KPI"] == kpi)). \
              select("META_FINAL")
        if df.count() != 0:
            results = [row[0] for row in df.select("META_FINAL").collect()]
            result = results[0]
    except Exception as e:
        logger.error(f"Error obteniendo meta por kpi. {e}")
    finally:
        return result


def get_execute_by_service(df: DataFrame, agent: str, period: str, segment: str) -> int:
    result = 0
    try:
        df = df.filter((df["AGENTE_COMISIONA"] == agent) & (df["PERIODO_PROCESO_CODIGO"] == period) &
                       (df["SEGMENTO"] == segment))
        result = df.count()
    except Exception as e:
        logger.error(f"Error obteniendo meta por segmento. {e}")
    finally:
        return result
