import time
import json
from typing import Any, Dict

from prefect import flow, get_run_logger

from Pipeline.CommissionProcess import CommissionProcess

SPARK_JARS = {
    "MONGO_CORE": "/opt/spark-jars/mongodb-driver-core-4.0.4.jar",
    "MONGO_CLIENT": "/opt/spark-jars/mongodb-driver-sync-4.0.4.jar",
    "MONGODB": "/opt/spark-jars/mongo-spark-connector_2.12-3.0.1.jar",
    "BSON": "/opt/spark-jars/bson-4.0.4.jar"
}

MONGODB_URI = "mongodb://bcom_spark_user:root@192.168.1.37:50001/bcom_spark"


@flow()
def run_commission(config: Dict[str, Any]) -> None:
    logger = get_run_logger()
    start_time = time.time()

    commission_process = CommissionProcess(config)

    # Conexion a Spark (LocalMode, StandAlone or Clúster)
    start_init = time.time()
    commission_process.init(SPARK_JARS, MONGODB_URI)
    logger.info(f"Duración de creación de sesión Spark: {time.time() - start_init}")

    # Primer task - Extraer la data - RECORDAR: SPARK ES LAZY!!!
    start_reader = time.time()
    commission_process.get_inputs(commission_process)
    logger.info(f"Duración de extracción de datos desde la BD: {time.time() - start_reader}")

    # Tercer task - Obtener metas
    start_process = time.time()
    goals = commission_process.get_goals_second_way(commission_process, "VENTAS", "GOALS")

    # Quinto task - Obtener ejecutados - ¿Aplicar tmb filtro de FLAG_COMISIONABLE y ACTIVE_USER_TRAFFIC?
    executes = commission_process.get_executed_second_way(commission_process, "VENTAS", "TEAMS")

    # Sexo task - Obtener monto origen
    base = commission_process.get_source_value(commission_process, "VENTAS", "COMERCIAL_BASE")

    result = commission_process.get_commission_per_agent(commission_process, goals, executes, base)
    logger.info(f"Duración de procesamiento en memoria: {time.time() - start_process}")

    # Task de escritura
    start_load = time.time()
    _ = commission_process.write_result(commission_process, result, "REPORT_SUMMARY")
    logger.info(f"Duración de carga del reporte a la BD: {time.time() - start_load}")

    logger.info(f"Duración de ejecución del proceso de etl: {time.time() - start_time}")


if __name__ == "__main__":

    conf_path = "config.json"
    with open(conf_path) as f:
        conf = json.load(f)

    # Run Commission
    run_commission(conf)


